@@ -1,5 +1,8 @@
 # frozen_string_literal: true
-ENV["RC_ARCHS"] = "" if RUBY_PLATFORM =~ /darwin/
+
+# rubocop:disable Style/GlobalVars
+
+ENV["RC_ARCHS"] = "" if RUBY_PLATFORM.include?("darwin")
 
 require "mkmf"
 require "rbconfig"
@@ -12,16 +15,14 @@ PACKAGE_ROOT_DIR = File.expand_path(File.join(File.dirname(__FILE__), "..", ".."
 REQUIRED_LIBXML_VERSION = "2.6.21"
 RECOMMENDED_LIBXML_VERSION = "2.9.3"
 
-# The gem version constraint in the Rakefile is not respected at install time.
-# Keep this version in sync with the one in the Rakefile !
-REQUIRED_MINI_PORTILE_VERSION = "~> 2.6.1"
+REQUIRED_MINI_PORTILE_VERSION = "~> 2.8.0" # keep this version in sync with the one in the gemspec
 REQUIRED_PKG_CONFIG_VERSION = "~> 1.1"
 
 # Keep track of what versions of what libraries we build against
 OTHER_LIBRARY_VERSIONS = {}
 
 NOKOGIRI_HELP_MESSAGE = <<~HELP
-  USAGE: ruby #{$0} [options]
+  USAGE: ruby #{$PROGRAM_NAME} [options]
 
     Flags that are always valid:
 
@@ -175,23 +176,23 @@ def config_system_libraries?
 end
 
 def windows?
-  RbConfig::CONFIG["target_os"] =~ /mingw32|mswin/
+  RbConfig::CONFIG["target_os"].match?(/mingw|mswin/)
 end
 
 def solaris?
-  RbConfig::CONFIG["target_os"] =~ /solaris/
+  RbConfig::CONFIG["target_os"].include?("solaris")
 end
 
 def darwin?
-  RbConfig::CONFIG["target_os"] =~ /darwin/
+  RbConfig::CONFIG["target_os"].include?("darwin")
 end
 
 def openbsd?
-  RbConfig::CONFIG["target_os"] =~ /openbsd/
+  RbConfig::CONFIG["target_os"].include?("openbsd")
 end
 
 def aix?
-  RbConfig::CONFIG["target_os"] =~ /aix/
+  RbConfig::CONFIG["target_os"].include?("aix")
 end
 
 def nix?
@@ -210,6 +211,18 @@ def local_have_library(lib, func = nil, headers = nil)
   have_library(lib, func, headers) || have_library("lib#{lib}", func, headers)
 end
 
+def gnome_source
+  # As of 2022-02-20, some mirrors have expired SSL certificates. I'm able to retrieve from my home,
+  # but whatever host is resolved on the github actions workers see an expired cert.
+  #
+  # See https://github.com/sparklemotion/nokogiri/runs/5266206403?check_suite_focus=true
+  if ENV["NOKOGIRI_USE_CANONICAL_GNOME_SOURCE"]
+    "https://download.gnome.org"
+  else
+    "https://mirror.csclub.uwaterloo.ca/gnome" # old reliable
+  end
+end
+
 LOCAL_PACKAGE_RESPONSE = Object.new
 def LOCAL_PACKAGE_RESPONSE.%(package)
   package ? "yes: #{package}" : "no"
@@ -284,17 +297,16 @@ ensure
 end
 
 def abort_could_not_find_library(lib)
-  abort("-----\n#{caller[0]}\n#{lib} is missing. Please locate mkmf.log to investigate how it is failing.\n-----")
+  callers = caller(1..2).join("\n")
+  abort("-----\n#{callers}\n#{lib} is missing. Please locate mkmf.log to investigate how it is failing.\n-----")
 end
 
-def chdir_for_build
+def chdir_for_build(&block)
   # When using rake-compiler-dock on Windows, the underlying Virtualbox shared
   # folders don't support symlinks, but libiconv expects it for a build on
   # Linux. We work around this limitation by using the temp dir for cooking.
-  build_dir = ENV["RCD_HOST_RUBY_PLATFORM"].to_s =~ /mingw|mswin|cygwin/ ? "/tmp" : "."
-  Dir.chdir(build_dir) do
-    yield
-  end
+  build_dir = /mingw|mswin|cygwin/.match?(ENV["RCD_HOST_RUBY_PLATFORM"].to_s) ? "/tmp" : "."
+  Dir.chdir(build_dir, &block)
 end
 
 def sh_export_path(path)
@@ -370,6 +382,7 @@ def iconv_configure_flags
   ["iconv", "opt"].each do |target|
     config = preserving_globals { dir_config(target) }
     next unless config.any? && try_link_iconv("--with-#{target}-* flags") { dir_config(target) }
+
     idirs, ldirs = config.map do |dirs|
       Array(dirs).flat_map do |dir|
         dir.split(File::PATH_SEPARATOR)
@@ -402,9 +415,9 @@ def iconv_configure_flags
   abort_could_not_find_library("libiconv")
 end
 
-def process_recipe(name, version, static_p, cross_p, cacheable_p=true)
+def process_recipe(name, version, static_p, cross_p, cacheable_p = true)
   require "rubygems"
-  gem("mini_portile2", REQUIRED_MINI_PORTILE_VERSION)
+  gem("mini_portile2", REQUIRED_MINI_PORTILE_VERSION) # gemspec is not respected at install time
   require "mini_portile2"
   message("Using mini_portile version #{MiniPortile::VERSION}\n")
 
@@ -413,9 +426,13 @@ def process_recipe(name, version, static_p, cross_p, cacheable_p=true)
   end
 
   MiniPortile.new(name, version).tap do |recipe|
+    def recipe.port_path
+      "#{@target}/#{RUBY_PLATFORM}/#{@name}/#{@version}"
+    end
+
     recipe.target = File.join(PACKAGE_ROOT_DIR, "ports") if cacheable_p
-    # Prefer host_alias over host in order to use i586-mingw32msvc as
-    # correct compiler prefix for cross build, but use host if not set.
+    # Prefer host_alias over host in order to use the correct compiler prefix for cross build, but
+    # use host if not set.
     recipe.host = RbConfig::CONFIG["host_alias"].empty? ? RbConfig::CONFIG["host"] : RbConfig::CONFIG["host_alias"]
     recipe.configure_options << "--libdir=#{File.join(recipe.path, "lib")}"
 
@@ -462,7 +479,7 @@ def process_recipe(name, version, static_p, cross_p, cacheable_p=true)
     end
 
     if RbConfig::CONFIG["target_cpu"] == "universal"
-      %w[CFLAGS LDFLAGS].each do |key|
+      ["CFLAGS", "LDFLAGS"].each do |key|
         unless env[key].include?("-arch")
           env[key] = concat_flags(env[key], RbConfig::CONFIG["ARCH_FLAG"])
         end
@@ -473,7 +490,7 @@ def process_recipe(name, version, static_p, cross_p, cacheable_p=true)
       "#{key}=#{value.strip}"
     end
 
-    checkpoint = "#{recipe.target}/#{recipe.name}-#{recipe.version}-#{recipe.host}.installed"
+    checkpoint = "#{recipe.target}/#{recipe.name}-#{recipe.version}-#{RUBY_PLATFORM}.installed"
     if File.exist?(checkpoint) && !recipe.source_directory
       message("Building Nokogiri with a packaged version of #{name}-#{version}.\n")
     else
@@ -487,7 +504,7 @@ def process_recipe(name, version, static_p, cross_p, cacheable_p=true)
         message("The following patches are being applied:\n")
 
         recipe.patch_files.each do |patch|
-          message("  - %s\n" % File.basename(patch))
+          message(format("  - %s\n", File.basename(patch)))
         end
       end
 
@@ -508,6 +525,7 @@ def process_recipe(name, version, static_p, cross_p, cacheable_p=true)
 
       EOM
 
+      pp(recipe.files)
       chdir_for_build { recipe.cook }
       FileUtils.touch(checkpoint)
     end
@@ -656,7 +674,7 @@ else
   if cross_build_p || windows?
     zlib_recipe = process_recipe("zlib", dependencies["zlib"]["version"], static_p, cross_build_p) do |recipe|
       recipe.files = [{
-        url: "http://zlib.net/fossils/#{recipe.name}-#{recipe.version}.tar.gz",
+        url: "https://zlib.net/fossils/#{recipe.name}-#{recipe.version}.tar.gz",
         sha256: dependencies["zlib"]["sha256"],
       }]
       if windows?
@@ -698,14 +716,6 @@ else
             execute("configure",
               ["env", "CHOST=#{host}", "CFLAGS=#{cflags}", "./configure", "--static", configure_prefix])
           end
-
-          def compile
-            if host =~ /darwin/
-              execute("compile", "make AR=#{host}-libtool")
-            else
-              super
-            end
-          end
         end
       end
     end
@@ -714,10 +724,13 @@ else
       libiconv_recipe = process_recipe("libiconv", dependencies["libiconv"]["version"], static_p,
         cross_build_p) do |recipe|
         recipe.files = [{
-          url: "http://ftp.gnu.org/pub/gnu/libiconv/#{recipe.name}-#{recipe.version}.tar.gz",
+          url: "https://ftp.gnu.org/pub/gnu/libiconv/#{recipe.name}-#{recipe.version}.tar.gz",
           sha256: dependencies["libiconv"]["sha256"],
         }]
 
+        # The libiconv configure script doesn't accept "arm64" host string but "aarch64"
+        recipe.host = recipe.host.gsub("arm64-apple-darwin", "aarch64-apple-darwin")
+
         cflags = concat_flags(ENV["CFLAGS"], "-O2", "-U_FORTIFY_SOURCE", "-g")
 
         recipe.configure_options += [
@@ -741,12 +754,21 @@ else
       Tools" to open the developer site, download the installer for your OS
       version and run it.
       -----
-      EOM
+    EOM
   end
 
-  unless windows?
-    preserving_globals { local_have_library("z", "gzdopen", "zlib.h") } ||
-      abort("zlib is missing; necessary for building libxml2")
+  if zlib_recipe
+    append_cppflags("-I#{zlib_recipe.path}/include")
+    $LIBPATH = ["#{zlib_recipe.path}/lib"] | $LIBPATH
+    ensure_package_configuration(opt: "zlib", pc: "zlib", lib: "z",
+      headers: "zlib.h", func: "gzdopen")
+  end
+
+  if libiconv_recipe
+    append_cppflags("-I#{libiconv_recipe.path}/include")
+    $LIBPATH = ["#{libiconv_recipe.path}/lib"] | $LIBPATH
+    ensure_package_configuration(opt: "iconv", pc: "iconv", lib: "iconv",
+      headers: "iconv.h", func: "iconv_open")
   end
 
   libxml2_recipe = process_recipe("libxml2", dependencies["libxml2"]["version"], static_p, cross_build_p) do |recipe|
@@ -754,8 +776,9 @@ else
     if source_dir
       recipe.source_directory = source_dir
     else
+      minor_version = Gem::Version.new(recipe.version).segments.take(2).join(".")
       recipe.files = [{
-        url: "http://xmlsoft.org/sources/#{recipe.name}-#{recipe.version}.tar.gz",
+        url: "#{gnome_source}/sources/libxml2/#{minor_version}/#{recipe.name}-#{recipe.version}.tar.xz",
         sha256: dependencies["libxml2"]["sha256"],
       }]
       recipe.patch_files = Dir[File.join(PACKAGE_ROOT_DIR, "patches", "libxml2", "*.patch")].sort
@@ -765,7 +788,6 @@ else
 
     if zlib_recipe
       recipe.configure_options << "--with-zlib=#{zlib_recipe.path}"
-      cflags = concat_flags(cflags, "-I#{zlib_recipe.path}/include")
     end
 
     if libiconv_recipe
@@ -803,8 +825,9 @@ else
     if source_dir
       recipe.source_directory = source_dir
     else
+      minor_version = Gem::Version.new(recipe.version).segments.take(2).join(".")
       recipe.files = [{
-        url: "http://xmlsoft.org/sources/#{recipe.name}-#{recipe.version}.tar.gz",
+        url: "#{gnome_source}/sources/libxslt/#{minor_version}/#{recipe.name}-#{recipe.version}.tar.xz",
         sha256: dependencies["libxslt"]["sha256"],
       }]
       recipe.patch_files = Dir[File.join(PACKAGE_ROOT_DIR, "patches", "libxslt", "*.patch")].sort
@@ -834,9 +857,6 @@ else
   append_cppflags("-DNOKOGIRI_PACKAGED_LIBRARIES")
   append_cppflags("-DNOKOGIRI_PRECOMPILED_LIBRARIES") if cross_build_p
 
-  $LIBPATH = ["#{zlib_recipe.path}/lib"] | $LIBPATH if zlib_recipe
-  $LIBPATH = ["#{libiconv_recipe.path}/lib"] | $LIBPATH if libiconv_recipe
-
   $libs = $libs.shellsplit.tap do |libs|
     [libxml2_recipe, libxslt_recipe].each do |recipe|
       libname = recipe.name[/\Alib(.+)\z/, 1]
@@ -929,7 +949,7 @@ libgumbo_recipe = process_recipe("libgumbo", "1.0.0-nokogiri", static_p, cross_b
 
       env = { "CC" => gcc_cmd, "CFLAGS" => cflags }
       if config_cross_build?
-        if host =~ /darwin/
+        if /darwin/.match?(host)
           env["AR"] = "#{host}-libtool"
           env["ARFLAGS"] = "-o"
         else
@@ -953,6 +973,8 @@ have_func("xmlRelaxNGSetParserStructuredErrors") # introduced in libxml 2.6.24
 have_func("xmlRelaxNGSetValidStructuredErrors") # introduced in libxml 2.6.21
 have_func("xmlSchemaSetValidStructuredErrors") # introduced in libxml 2.6.23
 have_func("xmlSchemaSetParserStructuredErrors") # introduced in libxml 2.6.23
+have_func("rb_gc_location") # introduced in Ruby 2.7
+have_func("rb_category_warning") # introduced in Ruby 3.0
 
 have_func("vasprintf")
 
